@echo off


REM=======================================================
REM              Windows 10 SSD T.R.I.M. 
REM              4-17-2016
REM              Zack Bozin
REM=======================================================

REM=======================================================
REM              Description
REM
REM  This script checks the SSD TRIM Status in Windows
REM  If Value = 1 TRIM is Disabled, if 0 TRIM is Enabled
REM
REM=======================================================
REM  A Trim command (known as TRIM in the ATA command set, 
REM  and UNMAP in the SCSI command set) allows an operating 
REM  system to inform a solid-state drive (SSD) which blocks 
REM  of data are no longer considered in use and can be wiped internally.[1]
REM
REM  Trim was introduced soon after SSDs were introduced. Because low-level 
REM  operation of SSDs differs significantly from hard drives, the typical way 
REM  in which operating systems handle operations like deletes and formats resulted 
REM  in unanticipated progressive performance degradation of write operations on SSDs.[2] 
REM
REM  Trimming enables the SSD to handle garbage collection overhead, which would otherwise 
REM  significantly slow down future write operations to the involved blocks, in advance.[3]
REM
REM  Although tools to "reset" some drives to a fresh state were already available before the 
REM  introduction of trimming, they also delete all data on the drive, which makes them impractical 
REM  to use for ongoing optimization.[4] By 2014 many SSDs had internal idle/background garbage collection 
REM  mechanisms that work independently of trimming; although this successfully maintains their performance 
REM  even under operating systems that do not support Trim, it has the associated drawbacks of increased 
REM  write amplification and wear of the flash cells.[5]
REM
REM  Source: https://en.wikipedia.org/wiki/Trim_(computing)
REM=======================================================

:check_Permissions
REM Check for Admin rights
net session >nul 2>&1
if %errorLevel% == 0 (
    GOTO continue
) else (
    ECHO.
    ECHO Please re-run this script as an Administrator.
)
pause >nul
GOTO :EOF

:continue

:header
@ECHO off
COLOR 8F
CLS
ECHO.
ECHO ======================================================================
ECHO =====                   Windows 10 SSD T.R.I.M.                  =====
ECHO =====                                                            =====
ECHO ===== This script checks if your SSD has T.R.I.M. enabled and    =====
ECHO ===== allows you to enable or disable T.R.I.M. on your SSD       =====
ECHO =====                  By Zack Bozin - 4/20/2016                 =====
ECHO ======================================================================
ECHO.
ECHO  1 = Check SSD T.R.I.M. Status
ECHO  2 = Enable SSD T.R.I.M.
ECHO  3 = Disable SSD T.R.I.M.
ECHO.
CHOICE /C 123 /N /M "Please enter your choice: "   
IF errorlevel 3 goto disable
IF errorlevel 2 goto enable
IF errorlevel 1 goto check



:check
ECHO If DisableDeleteNotify=0, then this means that Trim is enabled (that's a good thing)
ECHO.
fsutil behavior query DisableDeleteNotify
pause
GOTO header

:enable
fsutil behavior set DisableDeleteNotify 0
ECHO T.R.I.M. is now Enabled on your SSD
pause
GOTO header

:disable
fsutil behavior set DisableDeleteNotify 1
ECHO T.R.I.M. is now Disabled on your SSD
pause
GOTO header
