﻿#======================================================
#  Fix Name: Windows Repair Script
#  Version #: 1.0
#  Date Created: 4/22/2016
#  Creator Name: Cory Ebert
#  Contact Info: coebert@microsoft.com
#  Supported Operating Systems: Windows 8.1, Windows 10
#======================================================

#======================================================
#              Description
#
# This script self-elevates itself and then performs the
# following:
# 
# 1) Quick Fix- Uses function FirstTask: performs a series of dll reregistrations, resets clock and powerschemes, repairs app store and reregisters all system apps. 
# 2) Comprehensive Fix - Same as Quick Fix with addition of a DISM based RestoreHealth repair and a full SFC /Scannow. 
#======================================================


#======================================================
#              Functions
#======================================================
        function Quickfix
        (
            [string] $inputVariable
        )
        {
            # Import Assemblies

[void][reflection.assembly]::Load("System.Windows.Forms, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089")

#Supressing Errors (file not found,does not exist)
    $ErrorActionPreference = "SilentlyContinue"
# resyncing the time and date
    Start-Service W32Time
        & w32tm /resync
            Get-Date -format r
                Write-Host "Is this correct? If not, set time manually prior to continuing." -ForegroundColor Cyan
                    pause

# stop Windows Update Components
    Stop-Service -Force BITS|Out-Null
        Stop-Service -Force wuauserv|Out-Null
            Stop-Service -Force appidsvc|Out-Null
                Stop-Service -Force CryptSvc|Out-Null
# Delete the qmgr*.dat files

Remove-Item -Recurse "$env:ALLUSERSPROFILE\Application Data\Microsoft\Network\Downloader\*"

# Removing SoftwareDistribution
Remove-Item -recurse "$env:SystemRoot\winsxs\pending.xml"
CMD /C "rd /s /q $env:SystemRoot\SoftwareDistribution"
CMD /C "rd /s /q $env:SYSTEMRoot\system32\Catroot2"
Remove-Item -recurse "$env:SYSTEMROOT\WindowsUpdate.log"
# Reset the BITS service and the Windows Update service to the default
    CMD /C "sc.exe sdset bits D:(A;;CCLCSWRPWPDTLOCRRC;;;SY)(A;;CCDCLCSWRPWPDTLOCRSDRCWDWO;;;BA)(A;;CCLCSWLOCRRC;;;AU)(A;;CCLCSWRPWPDTLOCRRC;;;PU)"
        CMD /C "sc.exe sdset wuauserv D:(A;;CCLCSWRPWPDTLOCRRC;;;SY)(A;;CCDCLCSWRPWPDTLOCRSDRCWDWO;;;BA)(A;;CCLCSWLOCRRC;;;AU)(A;;CCLCSWRPWPDTLOCRRC;;;PU)"
# Reregister the BITS files and the Windows Update files
    Set-Location "$env:WINDIR\system32"
        & regsvr32.exe /s atl.dll
            & regsvr32.exe /s urlmon.dll
                & regsvr32.exe /s mshtml.dll
                    & regsvr32.exe /s shdocvw.dll
                        & regsvr32.exe /s browseui.dll
                            & regsvr32.exe /s jscript.dll
                                & regsvr32.exe /s vbscript.dll
                                    & regsvr32.exe /s scrrun.dll
                                        & regsvr32.exe /s msxml.dll
                                            & regsvr32.exe /s msxml3.dll
                                                & regsvr32.exe /s msxml6.dll
                                                    & regsvr32.exe /s actxprxy.dll
                                                & regsvr32.exe /s softpub.dll
                                            & regsvr32.exe /s wintrust.dll
                                        & regsvr32.exe /s dssenh.dll
                                    & regsvr32.exe /s rsaenh.dll
                                & regsvr32.exe /s gpkcsp.dll
                            & regsvr32.exe /s sccbase.dll
                        & regsvr32.exe /s slbcsp.dll
                    & regsvr32.exe /s cryptdlg.dll
                & regsvr32.exe /s oleaut32.dll
            & regsvr32.exe /s ole32.dll
        & regsvr32.exe /s shell32.dll
    & regsvr32.exe /s initpki.dll
& regsvr32.exe /s wuapi.dll
    & regsvr32.exe /s wuaueng.dll
        & regsvr32.exe /s wuaueng1.dll
            & regsvr32.exe /s wucltui.dll
                & regsvr32.exe /s wups.dll
                    & regsvr32.exe /s wups2.dll
                        & regsvr32.exe /s wuweb.dll
                            & regsvr32.exe /s qmgr.dll
                                & regsvr32.exe /s qmgrprxy.dll
                                    & regsvr32.exe /s wucltux.dll
                                        & regsvr32.exe /s muweb.dll
                                            & regsvr32.exe /s wuwebv.dll


# Reinstall System apps
Get-appxpackage -packageType bundle |% {add-appxpackage -register -disabledevelopmentmode ($_.installlocation + "\appxmetadata\appxbundlemanifest.xml")} | Out-Default; Clear-Host;
$bundlefamilies = (get-appxpackage -packagetype Bundle).packagefamilyname
get-appxpackage -packagetype main |? {-not ($bundlefamilies -contains $_.packagefamilyname)} |% {add-appxpackage -register -disabledevelopmentmode ($_.installlocation + "\appxmanifest.xml")} | Out-Default; Clear-Host;

# Resetting Winsock
    & netsh winsock reset
        & netsh advfirewall reset
            & ipconfig /flushdns
                & netsh winhttp reset proxy
# Starting the Windows Update services
    Start-Service BITS
        Start-Service wuauserv
            Start-Service cryptsvc
                Start-Service appidsvc
# Reauth

wuauclt.exe /resetauthorization /detectnow

# clear the store cache
    & wsreset.exe
     Start-Sleep -Seconds 3
    Stop-Process -processname WinStore.Mobile
    schtasks /run /tn "\Microsoft\Windows\WindowsUpdate\Automatic App Update" | Out-Null
# dism check for system corruption
   
    Repair-WindowsImage -Online -CheckHealth

# restore default power schemes
    & powercfg -restoredefaultschemes

# rebooting device

[void][System.Windows.Forms.MessageBox]::Show("Script is complete, after clicking ok, system will reboot. After reboot, please run Windows Update.")

Restart-Computer

 + $inputVariable
        }

        function ComprehensiveFix
        {    
            # Import Assemblies

[void][reflection.assembly]::Load("System.Windows.Forms, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089")

#Supressing Errors (file not found,does not exist)
    $ErrorActionPreference = "SilentlyContinue"

# resyncing the time and date
    Start-Service W32Time
        & w32tm /resync
            Get-Date -format r
                Write-Host "Is this correct? If not, set time manually prior to continuing." -ForegroundColor Cyan
                    pause
# stop Windows Update Components
    Stop-Service -Force BITS|Out-Null
        Stop-Service -Force wuauserv|Out-Null
            Stop-Service -Force appidsvc|Out-Null
                Stop-Service -Force CryptSvc|Out-Null
# Delete the qmgr*.dat files

Remove-Item -Recurse "$env:ALLUSERSPROFILE\Application Data\Microsoft\Network\Downloader\*"

# Removing SoftwareDistribution
Remove-Item -recurse "$env:SystemRoot\winsxs\pending.xml"
CMD /C "rd /s /q $env:SystemRoot\SoftwareDistribution"
CMD /C "rd /s /q $env:SYSTEMRoot\system32\Catroot2"
Remove-Item -recurse "$env:SYSTEMROOT\WindowsUpdate.log"
# Reset the BITS service and the Windows Update service to the default
    CMD /C "sc.exe sdset bits D:(A;;CCLCSWRPWPDTLOCRRC;;;SY)(A;;CCDCLCSWRPWPDTLOCRSDRCWDWO;;;BA)(A;;CCLCSWLOCRRC;;;AU)(A;;CCLCSWRPWPDTLOCRRC;;;PU)"
        CMD /C "sc.exe sdset wuauserv D:(A;;CCLCSWRPWPDTLOCRRC;;;SY)(A;;CCDCLCSWRPWPDTLOCRSDRCWDWO;;;BA)(A;;CCLCSWLOCRRC;;;AU)(A;;CCLCSWRPWPDTLOCRRC;;;PU)"
# Reregister the BITS files and the Windows Update files
    Set-Location "$env:WINDIR\system32"
        & regsvr32.exe /s atl.dll
            & regsvr32.exe /s urlmon.dll
                & regsvr32.exe /s mshtml.dll
                    & regsvr32.exe /s shdocvw.dll
                        & regsvr32.exe /s browseui.dll
                            & regsvr32.exe /s jscript.dll
                                & regsvr32.exe /s vbscript.dll
                                    & regsvr32.exe /s scrrun.dll
                                        & regsvr32.exe /s msxml.dll
                                            & regsvr32.exe /s msxml3.dll
                                                & regsvr32.exe /s msxml6.dll
                                                    & regsvr32.exe /s actxprxy.dll
                                                & regsvr32.exe /s softpub.dll
                                            & regsvr32.exe /s wintrust.dll
                                        & regsvr32.exe /s dssenh.dll
                                    & regsvr32.exe /s rsaenh.dll
                                & regsvr32.exe /s gpkcsp.dll
                            & regsvr32.exe /s sccbase.dll
                        & regsvr32.exe /s slbcsp.dll
                    & regsvr32.exe /s cryptdlg.dll
                & regsvr32.exe /s oleaut32.dll
            & regsvr32.exe /s ole32.dll
        & regsvr32.exe /s shell32.dll
    & regsvr32.exe /s initpki.dll
& regsvr32.exe /s wuapi.dll
    & regsvr32.exe /s wuaueng.dll
        & regsvr32.exe /s wuaueng1.dll
            & regsvr32.exe /s wucltui.dll
                & regsvr32.exe /s wups.dll
                    & regsvr32.exe /s wups2.dll
                        & regsvr32.exe /s wuweb.dll
                            & regsvr32.exe /s qmgr.dll
                                & regsvr32.exe /s qmgrprxy.dll
                                    & regsvr32.exe /s wucltux.dll
                                        & regsvr32.exe /s muweb.dll
                                            & regsvr32.exe /s wuwebv.dll
# Reinstall System apps
Get-appxpackage -packageType bundle |% {add-appxpackage -register -disabledevelopmentmode ($_.installlocation + "\appxmetadata\appxbundlemanifest.xml")} | Out-Default; Clear-Host;
$bundlefamilies = (get-appxpackage -packagetype Bundle).packagefamilyname
get-appxpackage -packagetype main |? {-not ($bundlefamilies -contains $_.packagefamilyname)} |% {add-appxpackage -register -disabledevelopmentmode ($_.installlocation + "\appxmanifest.xml")} | Out-Default; Clear-Host;

# Resetting Winsock
    & netsh winsock reset
        & netsh advfirewall reset
            & ipconfig /flushdns
                & netsh winhttp reset proxy
# Starting the Windows Update services
    Start-Service BITS
        Start-Service wuauserv
            Start-Service cryptsvc
                Start-Service appidsvc
# Reauthorize
wuauclt.exe /resetauthorization /detectnow

# clear the store cache and update apps
    & wsreset.exe
    Start-Sleep -Seconds 3
    Stop-Process -processname WinStore.Mobile

    schtasks /run /tn "\Microsoft\Windows\WindowsUpdate\Automatic App Update" | Out-Null

# dism check for system corruption
   
   CMD /C Dism /online /cleanup-image /startcomponentcleanup /resetbase |Out-Default |Clear-Host
   
    Repair-WindowsImage -Online -RestoreHealth
    Write-Host " DISM is finishing repair. Script will start SFC scan soon" -ForegroundColor Cyan
    SFC -scannow

# restore default power schemes
    & powercfg -restoredefaultschemes


# rebooting device

[void][System.Windows.Forms.MessageBox]::Show("Script is complete, after clicking ok, system will reboot. After reboot, please run Windows Update.")


Restart-Computer


        }
        function Use-RunAs 
{    
    # Check if script is running as Adminstrator and if not use RunAs 
    # Use Check Switch to check if admin 
     
    param([Switch]$Check) 
     
    $IsAdmin = ([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()` 
        ).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator") 
         
    if ($Check) { return $IsAdmin }     
 
    if ($MyInvocation.ScriptName -ne "") 
    {  
        if (-not $IsAdmin)  
        {  
            try 
            {  
                $arg = "-file `"$($MyInvocation.ScriptName)`"" 
                Start-Process "$psHome\powershell.exe" -Verb Runas -ArgumentList $arg -ErrorAction 'stop'  
            } 
            catch 
            { 
                Write-Warning "Error - Failed to restart script with runas"  
                break               
            } 
            exit # Quit this session of powershell 
        }  
    }  
    else  
    {  
        Write-Warning "Error - Script must be saved as a .ps1 file first"  
        break  
    }  
} 


#endregion
#======================================================
#              Elevate to Administrator
#======================================================

#Check for admin and elevate.
Use-RunAs

#======================================================
#                    Perform Task
#======================================================

#region Showing Options
Write-Host -ForegroundColor Yellow "Welcome to the Windows Repair Script"
""
""
""
Start-Sleep -Seconds 1
Write-Host -ForegroundColor Yellow "Choose a task:"
""
""

Write-Host -ForegroundColor Yellow "1. Run the quick fix (fixes general issues, does not attempt to repair OS corruption)"
"
"
Write-Host -ForegroundColor Yellow "2. Run the comprehensive fix (includes attempts to repair system corruption)" 
""
""
""
#endregion

#region Getting input
    While ($Valg -ne "1" -or $Valg -ne "2") {

    $Valg = Read-Host "Choose a number from the task list"

    If ($Valg –eq "1") { Break }
    If ($Valg –eq "2") { Break }

    if ($Valg -ne "1" -or $Valg -ne "2" ) {
        ""
        Write-Host -ForegroundColor Red "Oops. Try again..."
    }
}
#endregion

#region Main Code

    switch ($Valg) 
        { 
            1{
                QuickFix;
                break;
            } 
            2 {
                ComprehensiveFix;
                break;
            } 
        }
#endregion